// ShowResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "PhotoRedater.h"
#include "ShowResultsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// ShowResultsDlg dialog


const int kTimerID = 1234;

const TCHAR * ShowResultsDlg::kShowModeNames[SM_MAX] =
{
	_T("Batch Commands Created"),
	_T("Files Renamed")
};


ShowResultsDlg::ShowResultsDlg(ShowModeEnum ShowMode, CWnd* pParent /*=NULL*/)
	: CDialog(ShowResultsDlg::IDD, pParent), m_ShowMode(ShowMode)
{
	//{{AFX_DATA_INIT(ShowResultsDlg)
	m_ResultString = _T("");
	//}}AFX_DATA_INIT
	m_TimerHandle = 0;
	m_FilesAdded = 0;
	m_FilesProcessed = 0;
}


ShowResultsDlg::~ShowResultsDlg()
{
}


void ShowResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ShowResultsDlg)
	DDX_Text(pDX, IDC_EDIT_RESULTS, m_ResultString);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(ShowResultsDlg, CDialog)
	//{{AFX_MSG_MAP(ShowResultsDlg)
	ON_WM_TIMER()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ShowResultsDlg message handlers

BOOL ShowResultsDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_TimerHandle = SetTimer (kTimerID, 1 /* milliseconds */, NULL);

	return TRUE;
}


void ShowResultsDlg::OnTimer(UINT nIDEvent)
{
	DWORD CurrentTickCount;
	CEdit* pEditBox;
	DWORD ElapsedTime;
	CFileToRedate FileInfo;
	static DWORD LastTickCount = 0;
	static bool ScreenRefreshNeeded = false;
	CString WindowTitleString;

	if (m_TimerHandle != nIDEvent)
		CDialog::OnTimer(nIDEvent); // Someone else's timer.
	else // Process one work unit.
	{
		if (!m_FileNamesList.IsEmpty())
		{
			FileInfo = m_FileNamesList.RemoveHead();
			ProcessOneFile (FileInfo);

			// Switch to the slow timer if there isn't any more work.
			if (m_FileNamesList.IsEmpty())
				m_TimerHandle = SetTimer (kTimerID, 1000 /* milliseconds */, NULL);

			ScreenRefreshNeeded = true;
		}

		// Redraw the screen if needed, but only a few times per second.

		if (ScreenRefreshNeeded)
		{
			CurrentTickCount = GetTickCount();
			ElapsedTime = CurrentTickCount - LastTickCount;
			if (ElapsedTime > 1000)
			{
				UpdateData (FALSE);
				if (!m_FileNamesList.IsEmpty())
				{
					// Set the cursor to the end of the text box, so you
					// can see the new file names scrolling in.  Though
					// when done, leave it at the beginning.

					pEditBox = (CEdit*) GetDlgItem(IDC_EDIT_RESULTS);
					if (pEditBox != NULL)
					{
						int Length;
						Length = pEditBox->GetWindowTextLength();
						pEditBox->SetSel(Length, Length, FALSE);
					}
				}

				// Update the window title bar with the percent done.

				if (m_FilesProcessed < m_FilesAdded)
					WindowTitleString.Format (
						_T("%d %s of %d; %d%% complete."),
						m_FilesProcessed,
						kShowModeNames[m_ShowMode],
						m_FilesAdded,
						m_FilesProcessed * 100 / m_FilesAdded);
				else
					WindowTitleString.Format (
						_T("All %d %s!"),
						m_FilesProcessed,
						kShowModeNames[m_ShowMode]);
				SetWindowText (WindowTitleString);

				LastTickCount = CurrentTickCount;
				ScreenRefreshNeeded = false;
			}
		}
	}
}


void ShowResultsDlg::OnClose()
{
	if (m_TimerHandle != 0)
		KillTimer (m_TimerHandle);
	m_TimerHandle = 0;

	CDialog::OnClose();
}


/////////////////////////////////////////////////////////////////////////////
// ShowResultsDlg other code.

void ShowResultsDlg::AddFileName(const TCHAR *PathName,
	const TCHAR *CameraName,
	CTimeSpan DeltaTime)
{
	CFileToRedate FileInfo;

	FileInfo.m_PathName = PathName;
	FileInfo.m_CameraName = CameraName;
	FileInfo.m_DeltaTime = DeltaTime;

	m_FileNamesList.AddTail(FileInfo);
	m_FilesAdded++;
}


void ShowResultsDlg::AppendText(LPCTSTR NewText)
{
	m_ResultString += NewText;
}


void ShowResultsDlg::ProcessOneFile(CFileToRedate &FileInfo)
{
	CString Extension;
	CString FileName;
	CFileStatus FileStatus;
	int Index;
	int IndexEnd;
	TCHAR Letter;
	CString RenamedFileName;
	CString RenamedPathName;
	CString OutString;
	CString Name;
	CString Path;
	CString Sequence;

	if (CFile::GetStatus(FileInfo.m_PathName, FileStatus))
	{
		CTime AdjustedTime;

		AdjustedTime = FileStatus.m_mtime + FileInfo.m_DeltaTime;

		// Extract the path from the file name.

		Index = FileInfo.m_PathName.ReverseFind('\\');
		if (Index < 0)
			Index = FileInfo.m_PathName.ReverseFind('/');
		if (Index < 0)
			Index = FileInfo.m_PathName.ReverseFind(':');
		if (Index < 0)
		{
			Path = _T("");
			FileName = FileInfo.m_PathName;
		}
		else
		{
			Path = FileInfo.m_PathName.Left(Index+1); // Includes trailing slash.
			FileName = FileInfo.m_PathName.Mid(Index+1);
		}

		// Get the file extension.

		Index = FileName.ReverseFind('.');
		if (Index < 0)
		{
			Extension = _T("");
			Name = FileName;
		}
		else
		{
			Extension = FileName.Mid(Index); // Includes leading period.
			Name = FileName.Left(Index);
		}

		// Get the sequence number from the rear of the file name.  Will skip
		// over non-numeric characters after the last number in the name.

		for (Index = Name.GetLength()-1; Index >= 0; Index--)
		{
			Letter = Name.GetAt(Index);
			if (Letter >= '0' && Letter <= '9')
				break;
		}
		if (Index < 0)
			Sequence = _T("");
		else
		{
			IndexEnd = Index; // Points at last digit of sequence.
			while (Index >= 0)
			{
				Letter = Name.GetAt(Index);
				if (Letter < '0' || Letter > '9')
					break;
				Index--;
			}
			Index++; // Point at first digit of sequence.
			Sequence = Name.Mid(Index, IndexEnd - Index + 1);
			if (!Sequence.IsEmpty())
				Sequence = _T("_") + Sequence;
		}

		RenamedFileName.Format (_T("%s_%s%s%s"),
				AdjustedTime.Format(_T("%Y%m%d_%H%M%S")),
				FileInfo.m_CameraName,
				Sequence,
				Extension);
		RenamedPathName = Path + RenamedFileName;

		if (m_ShowMode == SM_DO_RENAME_OPERATION)
		{
			if (rename (FileInfo.m_PathName, RenamedPathName))
				OutString.Format (_T("Failed to rename \"%s\" to \"%s\", "
					"error code %08x (%s).\r\n"),
					FileInfo.m_PathName, RenamedPathName,
					errno, strerror (errno));
			else
				OutString.Format (_T("Renamed \"%s\" to %s.\r\n"),
					FileInfo.m_PathName, RenamedFileName);
			m_ResultString += OutString;
		}
		else
		{
			// Create a rename command, which doesn't have path names, so the
			// user can run it from the photo directory (rename seems to
			// require that in Windows).

			OutString.Format (_T("ren \"%s\" \"%s\"\r\n"),
				FileName, RenamedFileName);
			m_ResultString += OutString;

			// Create a copy command, which does have the full path names.

			OutString.Format (
				_T("REM copy \"%s\" \"%s%s\"\r\n\r\n"),
				FileInfo.m_PathName,
				Path,
				RenamedFileName);
			m_ResultString += OutString;
		}
	}
	else // File doesn't exist etc.
	{
		OutString.Format(
			_T("REM Skipping \"%s\", perhaps it doesn't exist?\r\n"),
			FileInfo.m_PathName);
		m_ResultString += OutString;
	}
	m_FilesProcessed++;
}
